<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://estatebud.com
 * @since      1.0.0
 *
 * @package    Estatebud
 * @subpackage Estatebud/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Estatebud
 * @subpackage Estatebud/admin
 * @author     Estatebud
 */
class Estatebud_Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $id    The ID of this plugin.
	 */
	private $id;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since      1.0.0
	 * @param      string    $id         The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($id, $version)
	{

		$this->id = $id;
		$this->version = $version;

		add_action('admin_menu', array($this, 'create_admin_settings_page'));
		add_action('admin_init', array($this, 'estatebud_email_template_settings'));
		add_filter('parent_file', array($this, 'estatebud_parent_file'));

		add_action("wp_ajax_estatebud_get_listing", array($this, "estatebud_get_listing"));
		add_action("wp_ajax_nopriv_estatebud_get_listing", array($this, "estatebud_get_listing"));

		add_action("wp_ajax_estatebud_send_contact", array($this, "estatebud_send_contact"));
		add_action("wp_ajax_nopriv_estatebud_send_contact", array($this, "estatebud_send_contact"));

		if (isset($_GET["tab"]) && $_GET["tab"] == "email-settings") {
			add_filter('tiny_mce_before_init', array($this, 'estatebud_override_mce_options'));
		}
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{
		wp_enqueue_style($this->id, plugin_dir_url(__FILE__) . 'css/estatebud-admin.css', array(), $this->version, 'all');
		wp_enqueue_style("jquery-select2", plugin_dir_url(__FILE__) . '../public/css/select2.css', array(), $this->version, 'all');
		if (isset($_GET["page"]) && $_GET["page"] == "estatebud_settings" && isset($_GET["tab"]) && $_GET["tab"] == "plugin-settings") {
			wp_enqueue_style('wp-color-picker');
		}
		if (isset($_GET["page"]) && $_GET["page"] == "estatebud_settings" && isset($_GET["tab"]) && ($_GET["tab"] == "shortcodes" || $_GET["tab"] == "widgets")) {
			wp_enqueue_style("bootstrap", plugin_dir_url(__FILE__) . 'css/grid.min.css', array(), $this->version, 'all');
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
		add_thickbox();

		wp_enqueue_script($this->id, plugin_dir_url(__FILE__) . 'js/estatebud-admin.js', array('jquery'), $this->version, false);
		wp_enqueue_script("jquery-select2", plugin_dir_url(__FILE__) . '../public/js/select2.full.min.js', array('jquery'), $this->version, false);

		if (isset($_GET["page"]) && $_GET["page"] == "estatebud_settings" && isset($_GET["tab"]) && $_GET["tab"] == "shortcodes") {
			wp_localize_script($this->id, 'estatebudL10n', array(
				'error' => __("Error: ", "estatebud"),
				'edit_form_title' => __('Edit Existing Shortcode', "estatebud"),
				'edit_form_button' => __('Save Shortcode', "estatebud"),
				'new_form_title' => __('Create New Shortcode', "estatebud"),
				'new_form_button' => __('Create Shortcode', "estatebud"),
				'new_form_security' => wp_create_nonce("estatebud_save_shortcode", "shortcode_security"),
				'edit_form_security' => wp_create_nonce("estatebud_edit_shortcode", "shortcode_security")
			));
		} else if (isset($_GET["page"]) && $_GET["page"] == "estatebud_settings" && isset($_GET["tab"]) && $_GET["tab"] == "widgets") {
			wp_enqueue_script( 'ace_code_highlighter_js', plugin_dir_url(__FILE__) . 'js/ace.js', '', '1.0.0', true );
        	wp_enqueue_script( 'ace_mode_js', plugin_dir_url(__FILE__) . 'js/mode-html.js', array( 'ace_code_highlighter_js' ), '1.0.0', true );
			wp_localize_script($this->id, 'estatebudL10n', array(
				'error' => __("Error: ", "estatebud"),
				'edit_form_title' => __('Edit Existing Widget', "estatebud"),
				'edit_form_button' => __('Save Widget', "estatebud"),
				'new_form_title' => __('Create New Widget', "estatebud"),
				'new_form_button' => __('Create Widget', "estatebud"),
				'new_form_security' => wp_create_nonce("estatebud_save_widget", "widget_security"),
				'edit_form_security' => wp_create_nonce("estatebud_edit_widget", "widget_security")
			));
		} else if (isset($_GET["page"]) && $_GET["page"] == "estatebud_settings" && isset($_GET["tab"]) && $_GET["tab"] == "plugin-settings") {
			wp_enqueue_script('wp-color-picker');
			wp_enqueue_script( 'ace_code_highlighter_js', plugin_dir_url(__FILE__) . 'js/ace.js', '', '1.0.0', true );
        	wp_enqueue_script( 'ace_mode_js', plugin_dir_url(__FILE__) . 'js/mode-css.js', array( 'ace_code_highlighter_js' ), '1.0.0', true );
		}
	}

	/**
	 * Creating the settings page
	 *
	 * @since    1.0.0
	 */
	public function create_admin_settings_page()
	{

		global $available_pages;

		$available_pages = [
			["slug" => "welcome", "title" => __("Welcome", "estatebud"), "extra_classes" => "nav-tab-estatebud-welcome", "icon" => "dashicons dashicons-admin-home"],
			["slug" => "plugin-settings", "title" => __("Plugin Settings", "estatebud"), "extra_classes" => "nav-tab-estatebud-plugin-settings", "icon" => "dashicons dashicons-admin-tools"],
			["slug" => "api-settings", "title" => __("API Settings", "estatebud"), "extra_classes" => "nav-tab-estatebud-api", "icon" => "dashicons dashicons-admin-generic"],
			["slug" => "listing-settings", "title" => __("Listing Settings", "estatebud"), "extra_classes" => "nav-tab-estatebud-api", "icon" => "dashicons dashicons-exerpt-view"],
			["slug" => "email-preferences", "title" => __("E-mail Preferences", "estatebud"), "extra_classes" => "nav-tab-estatebud-email", "icon" => "dashicons dashicons-email"],
			["slug" => "shortcodes", "title" => __("Shortcodes", "estatebud"), "extra_classes" => "nav-tab-estatebud-shortcodes", "icon" => "dashicons dashicons-welcome-view-site"],
			["slug" => "widgets", "title" => __("Search Boxes", "estatebud"), "extra_classes" => "nav-tab-estatebud-widgets", "icon" => "dashicons dashicons-search"],
			["slug" => "support", "title" => __("Support", "estatebud"), "extra_classes" => "nav-tab-estatebud-support", "icon" => "dashicons dashicons-sos"]
		];
		add_menu_page(
			__('Estatebud Settings', "estatebud"),			// The title to be displayed in the browser window for this page.
			__('Estatebud', "estatebud"),					// The text to be displayed for this menu item
			'administrator',								// Which type of users can see this menu item
			'estatebud_settings',							// The unique ID - that is, the slug - for this menu item
			array($this, 'estatebud_settings_page'),		// The name of the function to call when rendering this menu's page
			plugin_dir_url(__FILE__) . "assets/estatebud.svg"  // The icon path
		);
		foreach ($available_pages as $available_page) {
			add_submenu_page('estatebud_settings', $available_page["title"], $available_page["title"], "administrator", "admin.php?page=estatebud_settings&tab=" . $available_page["slug"]);
		}
		remove_submenu_page('estatebud_settings', 'estatebud_settings');
	}

	/**
	 * Fix for active class selection on the submenu pages
	 *
	 * @param    $parent_file
	 * @return   void
	 * @since 	 1.0.0
	 */
	function estatebud_parent_file($parent_file)
	{
		global $submenu_file, $available_pages;
		foreach ($available_pages as $available_page) {
			if (isset($_GET['tab']) && $_GET['tab'] == $available_page["slug"]) {
				$submenu_file = 'admin.php?page=estatebud_settings&tab=' . $available_page["slug"];
				break;
			}
		}
		return $parent_file;
	}

	/**
	 * Display admin settings page
	 *
	 * @since    1.0.0
	 */
	public function estatebud_settings_page()
	{
		require_once dirname(__FILE__) . "/partials/estatebud-admin-display.php";
	}

	/**
	 * Email Templates Section
	 *
	 * @return void
	 */
	function estatebud_email_template_settings()
	{
		// Section
		add_settings_section('estatebud_email_templates_section', 'Email Templates', array($this, 'estatebud_email_templates_section'), 'estatebud-email-templates');

		// Field
		add_settings_field(
			'estatebud_email_company_subject',
			__('Subject for Company Email', "estatebud"),
			array($this, 'email_template_company_subject_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_user_subject',
			__('Subject for User Email', "estatebud"),
			array($this, 'email_template_user_subject_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_privacy_policy_url',
			__('Privacy Policy URL', "estatebud"),
			array($this, 'email_template_privacy_policy_url_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_sender_name',
			__('Sender E-mail Name', "estatebud"),
			array($this, 'email_template_sender_name_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_sender_address',
			__('Sender E-mail Address', "estatebud"),
			array($this, 'email_template_sender_address_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_company_recipient',
			__('Recipient Address(es) for Company Email', "estatebud"),
			array($this, 'email_template_recipient_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_company_line',
			__('Company Info Text', "estatebud"),
			array($this, 'email_template_company_line_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_footer_line',
			__('Footer Line Text', "estatebud"),
			array($this, 'email_template_footer_line_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_template_user',
			__('E-mail sent to the user', "estatebud"),
			array($this, 'email_template_user_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Field
		add_settings_field(
			'estatebud_email_template_contact',
			__('E-mail sent to the company', "estatebud"),
			array($this, 'email_template_contact_cb'),
			'estatebud-email-templates',
			'estatebud_email_templates_section'
		);

		// Value under which we are saving the data by $_POST
		register_setting('estatebud-email-templates', 'estatebud_email_template_user');
		register_setting('estatebud-email-templates', 'estatebud_email_template_contact');
		register_setting('estatebud-email-templates', 'estatebud_email_footer_line');
		register_setting('estatebud-email-templates', 'estatebud_email_company_line');
		register_setting('estatebud-email-templates', 'estatebud_email_company_recipient');
		register_setting('estatebud-email-templates', 'estatebud_email_sender_address');
		register_setting('estatebud-email-templates', 'estatebud_email_sender_name');
		register_setting('estatebud-email-templates', 'estatebud_email_privacy_policy_url');
		register_setting('estatebud-email-templates', 'estatebud_email_company_subject');
		register_setting('estatebud-email-templates', 'estatebud_email_user_subject');

	}

	function estatebud_email_templates_section()
	{
		echo __('You can customize the e-mails sent both to the requester and the company when someone requests more information about a property via the contact form on the property page or from the widget.', "estatebud");
		echo "<br><br><br>";
		echo __('<div class="estatebud-info"><b>Available placeholders:</b>', "estatebud");
		echo "<br><br><table>";
		echo "<tr><td style='width:200px'><code>%name%</code></td><td>" . __('User\'s name.', "estatebud") . "</td>";
		echo "<td><code>%property_id%</code></td><td>" . __('The property ID which user enquired about.', "estatebud") . "</td></tr>";
		echo "<tr><td><code>%email%</code></td><td>" . __('The email of the enquiring user.', "estatebud") . "</td>";
		echo "<td><code>%website_name%</code></td><td>" . __('The website name which the form has been submitted.', "estatebud") . "</td></tr>";
		echo "<tr><td><code>%website_url%</code></td><td>" . __('The website URL which the form has been submitted.', "estatebud") . "</td>";
		echo "<td><code>%phone%</code></td><td>" . __('The user\'s phone number if entered.', "estatebud") . "</td></tr>";
		echo "<tr><td><code>%message%</code></td><td>" . __('The user\'s message.', "estatebud") . "</td>";
		echo "<td><code>%timestamp%</code></td><td>" . __('The date/time when the user submitted the form.', "estatebud") . "</td></tr>";
		echo "<tr><td><code>%company_info%</code></td><td>" . __('The company info input you enter on this form. Shown on footer.', "estatebud") . "</td>";
		echo "<td><code>%footer_text%</code></td><td>" . __('The footer input you enter on this form. Shown on footer.', "estatebud") . "</td></tr>";
		echo "</table>";
		echo "<br></div><br>";
	}

	function email_template_company_line_cb()
	{
		$content = sanitize_text_field(get_option('estatebud_email_company_line'));
		echo "<div class='form-field'><input type='text' name='estatebud_email_company_line' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("This will be on the footer of the email template where <code>%company_info%</code> placeholder exists.", "estatebud") . "</p>";
	}

	function email_template_user_subject_cb()
	{
		$content = sanitize_text_field(get_option('estatebud_email_user_subject'));
		echo "<div class='form-field'><input type='text' name='estatebud_email_user_subject' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("This will be the subject of the email which is sent to the contact requester.", "estatebud") . "</p>";
	}

	function email_template_company_subject_cb()
	{
		$content = sanitize_text_field(get_option('estatebud_email_company_subject'));
		echo "<div class='form-field'><input type='text' name='estatebud_email_company_subject' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("This will be the subject of the email which is sent to you.", "estatebud") . "</p>";
	}

	function email_template_recipient_cb()
	{
		$recipient = get_option('estatebud_email_company_recipient', false);
		if ($recipient == false) {
			$recipient = get_bloginfo("admin_email");
			update_option('estatebud_email_company_recipient', $recipient);
		}
		$content = sanitize_text_field($recipient);
		echo "<div class='form-field'><input type='text' name='estatebud_email_company_recipient' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("Seperate e-mail addresses with <code>;</code>", "estatebud") . "</p>";
	}

	function email_template_sender_name_cb()
	{
		$sender_name = get_option('estatebud_email_sender_name', false);
		if ($sender_name == false) {
			$sender_name = get_bloginfo("blogname");
			update_option('estatebud_email_sender_name', $sender_name);
		}
		$content = sanitize_text_field($sender_name);
		echo "<div class='form-field'><input type='text' name='estatebud_email_sender_name' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("Visible Sender Name of the e-mail", "estatebud") . "</p>";
	}

	function email_template_sender_address_cb()
	{
		$sender_address = get_option('estatebud_email_sender_address', false);
		if ($sender_address == false) {
			$sitename = strtolower( $_SERVER["SERVER_NAME"] );
			if ( substr( $sitename, 0, 4 ) == "www." ) {
				$sitename = substr( $sitename, 4 );
			}
			$sender_address = "wordpress@$sitename";
			update_option('estatebud_email_sender_address', $sender_address);
		}
		$content = sanitize_text_field($sender_address);
		echo "<div class='form-field'><input type='text' name='estatebud_email_sender_address' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("Visible Sender Address of the e-mail", "estatebud") . "</p>";
	}

	function email_template_privacy_policy_url_cb()
	{
		$pp_url = get_option('estatebud_email_privacy_policy_url', false);
		if ($pp_url == false) {
			$pp_url = "";
			update_option('estatebud_email_privacy_policy_url', $pp_url);
		}
		$content = sanitize_text_field($pp_url);
		echo "<div class='form-field'><input type='text' name='estatebud_email_privacy_policy_url' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("Privacy Policy document link which will be visible below the contact form if set", "estatebud") . "</p>";
	}

	function email_template_footer_line_cb()
	{
		$content = sanitize_text_field(get_option('estatebud_email_footer_line'));
		echo "<div class='form-field'><input type='text' name='estatebud_email_footer_line' value='" . $content . "'></div>";
		echo "<p class='description'>" . __("This will be on the footer of the email template where <code>%footer_text%</code> placeholder exists.", "estatebud") . "</p>";
	}

	function email_template_contact_cb()
	{
		$content = get_option('estatebud_email_template_contact');
		wp_editor($content, 'estatebud_email_template_contact');
	}

	function email_template_user_cb()
	{
		$content = get_option('estatebud_email_template_user');
		wp_editor($content, 'estatebud_email_template_user');
	}

	function estatebud_override_mce_options($initArray)
	{
		if (substr($initArray["selector"], 1, 9) == "estatebud") {
			$opts = '*[*]';
			$initArray['valid_elements'] = $opts;
			$initArray['extended_valid_elements'] = $opts;
			$initArray['remove_linebreaks'] = false;
			$initArray['remove_redundant_brs'] = false;
			$initArray['wpautop'] = false;
			return $initArray;
		}
	}

	/**
	 * Public AJAX callback to fetch property list
	 *
	 * @return void
	 */
	public function estatebud_get_listing()
	{
		check_ajax_referer("estatebud_get_listing", "security");

		$api_key = get_option("estatebud_api_key", null);

		if (!empty($api_key)) {
            $estatebud_api = new EstatebudAPI($api_key, isset($_GET["lang"]) ? $_GET["lang"] : get_locale());
			if ($estatebud_api->is_valid) {
				$data = $_GET;

				// fixes for empty selectboxes
				if(!isset($data["category"])) $data["category"] = [];
				if(!isset($data["status"])) $data["status"] = [];
				if(!isset($data["area"])) $data["area"] = [];
				if(!isset($data["type"])) $data["type"] = [];

				$shortcode_key = $_SERVER["HTTP_ESTATEBUD_SHORTCODE"];
				if($shortcode_key){
					$shortcodes = get_option("estatebud_shortcode_settings", []);
					if(count($shortcodes) && isset($shortcodes[$shortcode_key])){
						$data = array_merge($shortcodes[$shortcode_key], $data);
					}
				}
				$estatebud_list = $this->call_user_func_assoc_array(array($estatebud_api, "get_properties"), $data);
				require_once dirname(__FILE__) . "/../public/partials/estatebud-public-listing-properties.php";
			}
		}
		die();
	}

	private function call_user_func_assoc_array($callback, $args){
        $ref = new ReflectionMethod(get_class($callback[0]) . "::" . $callback[1]);
        $parameters = $ref->getParameters();
        $params = [];
        foreach($parameters as $parameter){
            $param_name = $parameter->getName();
            $params[$param_name] = isset($args[$param_name]) ? $args[$param_name] : $parameter->getDefaultValue();
            if(is_array($params[$param_name])) $params[$param_name] = implode(",", $params[$param_name]);
        }
        return call_user_func_array($callback, $params);
    }

	/**
	 * Public AJAX callback to do contact form actions
	 *
	 * @return void
	 */
	public function estatebud_send_contact()
	{
		remove_filter('wp_mail', 'wp_staticize_emoji_for_email');
		check_ajax_referer("estatebud_send_contact", "contact_security");

		$use_recaptcha = get_option("estatebud_use_recaptcha", false);
		$pp_url = trim(get_option("estatebud_email_privacy_policy_url", ""));
		$pp_url = empty($pp_url) ? false : $pp_url;
		$site_key = get_option("recaptcha_sitekey_value", false);
		$site_secret = get_option("estatebud_gcaptcha_secret", false);
		$email_sender_name = get_option("estatebud_email_sender_name", false);
		$email_sender_address = get_option("estatebud_email_sender_address", false);

		if($pp_url && !isset($_POST["estatebud-privacy-policy-accepted"])){
			die(json_encode([
				"status" => "error",
				"message" => __("Please indicate that you have read and agreed to the Privacy Policy", "estatebud")
			]));
		}

		if($use_recaptcha){
			if(empty($site_secret) || empty($site_key)){
				die(json_encode([
					"status" => "error",
					"message" => __("The Google reCAPTCHA is not installed. Please contact Site Administrator.", "estatebud")
				]));
			}else{
				if($this->verify_google_captcha($_POST["g-recaptcha-response"], $site_secret) === false){
					die(json_encode([
						"status" => "error",
						"message" => __("The Google reCAPTCHA verification failed. Please try again.", "estatebud")
					]));
				}
			}
		}

		// send user mail
		$to = sanitize_email($_POST['email']);
		$subject = get_option("estatebud_email_user_subject", null);
		$body = get_option("estatebud_email_template_user", null);
		$body = str_replace("%name%", sanitize_text_field($_POST["name"]), $body);
		$body = str_replace("%email%", sanitize_email($_POST["email"]), $body);
		$body = str_replace("%phone%", sanitize_text_field($_POST["phone"]), $body);
		$body = str_replace("%property_id%", sanitize_text_field($_POST["property_id"]), $body);
		$body = str_replace("%website_url%", sanitize_text_field(home_url()), $body);
		$body = str_replace("%website_name%", sanitize_text_field(get_bloginfo("site_name")), $body);
		$body = str_replace("%message%", sanitize_textarea_field($_POST["message"]), $body);
		$body = str_replace("%timestamp%", current_time('mysql'), $body);
		$body = str_replace("%footer_text%", get_option("estatebud_email_footer_line"), $body);
		$body = str_replace("%company_info%", get_option("estatebud_email_company_line"), $body);
		$body = "<style>.aligncenter { margin: 0 auto; text-align: center; }</style>" . $body;
		$headers = array('Content-Type: text/html; charset=UTF-8');

		if($email_sender_name && $email_sender_address && !empty($email_sender_name) && !empty($email_sender_address) && is_email($email_sender_address)){
			$headers[] = 'From: '.$email_sender_name.' <'.$email_sender_address.'>';
		}

		$result = wp_mail($to, $subject, $body, $headers);

		if ($result) {
			// send company mail
			$to = [sanitize_email(get_bloginfo("admin_email"))];
			$others = array_map("trim", explode(";", get_option("estatebud_email_company_recipient")));
			foreach ($others as $other) {
				$to[] = sanitize_email($other);
			}
			$to = array_unique($to);
			$subject = get_option("estatebud_email_company_subject", null);
			$body = get_option("estatebud_email_template_contact", null);
			$body = str_replace("%name%", sanitize_text_field($_POST["name"]), $body);
			$body = str_replace("%email%", sanitize_email($_POST["email"]), $body);
			$body = str_replace("%phone%", sanitize_text_field($_POST["phone"]), $body);
			$body = str_replace("%property_id%", sanitize_text_field($_POST["property_id"]), $body);
			$body = str_replace("%website_url%", sanitize_text_field(home_url()), $body);
			$body = str_replace("%website_name%", sanitize_text_field(get_bloginfo("site_name")), $body);
			$body = str_replace("%message%", sanitize_textarea_field($_POST["message"]), $body);
			$body = str_replace("%timestamp%", current_time('mysql'), $body);
			$body = str_replace("%footer_text%", get_option("estatebud_email_footer_line"), $body);
			$body = str_replace("%company_info%", get_option("estatebud_email_company_line"), $body);
			$body = "<style>.aligncenter { margin: 0 auto; text-align: center; }</style>" . $body;
			$headers = array('Content-Type: text/html; charset=UTF-8');

			if($email_sender_name && $email_sender_address && !empty($email_sender_name) && !empty($email_sender_address) && is_email($email_sender_address)){
				$headers[] = 'From: '.$email_sender_name.' <'.$email_sender_address.'>';
			}

			$result = wp_mail($to, $subject, $body, $headers);
		}

		if ($result) {
			echo json_encode(["status" => "success", "message" => __("Your contact request is successfully sent. We will respond to it as soon as we receive the details. Thank you for contacting us.", "estatebud")]);
		} else {
			echo json_encode(["status" => "error", "message" => __("Oops! Something went wrong. Please try again.", "estatebud")]);
		}
		die();
	}

	private function verify_google_captcha($gcaptcha_response, $secret)
	{
		//open a new curl connection
		$ch = curl_init();
		$url = "https://www.google.com/recaptcha/api/siteverify";
		$fields = array(
			"secret" => $secret,
			"response" => $gcaptcha_response,
		);
		$fields_string = "";
		foreach ($fields as $key => $value) {
			$fields_string .= $key . '=' . $value . '&';
		}
		rtrim($fields_string, '&');
		//set the url, number of POST vars, POST data
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_POST, count($fields));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		//execute post request
		$result = curl_exec($ch);
		//close connection
		curl_close($ch);
		$json_data = json_decode($result);
		//If the captcha was declined, redirect the user to the same page with the query var "captcha" set to "not-entered"
		if (!$json_data || $json_data->success == false) {
			return false;
		}
		return true;
	}
}