<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://estatebud.com
 * @since             1.0.0
 * @package           Estatebud
 *
 * @wordpress-plugin
 * Plugin Name:       Estatebud - Properties & Listings
 * Description:       Connects your WordPress website with Estatebud to display realtime property listings.
 * Version:           2.6.1
 * Author:            Estatebud
 * Author URI:        http://estatebud.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       estatebud
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
	die;
}

/**
 * Current plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define('ESTATEBUD_PLUGIN_VERSION', '2.6.1');


/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-estatebud-activator.php
 */
function activate_estatebud()
{
	require_once plugin_dir_path(__FILE__) . 'includes/class-estatebud-activator.php';
	Estatebud_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-estatebud-deactivator.php
 */
function deactivate_estatebud()
{
	require_once plugin_dir_path(__FILE__) . 'includes/class-estatebud-deactivator.php';
	Estatebud_Deactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_estatebud');
register_deactivation_hook(__FILE__, 'deactivate_estatebud');

/**
 * Settings link in plugin management screen
 */
function estatebud_plugin_settings_link($actions, $file)
{
	if (false !== strpos($file, 'estatebud'))
		$actions['settings'] = '<a href="admin.php?page=estatebud_settings">'. __("Settings", "estatebud") . '</a>';
	return $actions;
}

add_filter('plugin_action_links', 'estatebud_plugin_settings_link', 2, 2);

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/class-estatebud.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_estatebud()
{
	$plugin = new Estatebud();
	$plugin->run();
}
run_estatebud();

/**
 * Ajax Method to save the shortcode data
 *
 * @since 	 1.0.0
 */
function estatebud_save_shortcode()
{
	if (current_user_can('manage_options') && !empty($_POST) && check_admin_referer('estatebud_save_shortcode', 'shortcode_security')) {
		$values = get_option("estatebud_shortcode_settings", []);
		if (!is_array($values)) $values = [];
		if (isset($values[$_POST["shortcode_key"]])) {
			die(json_encode(["status" => "error", "message" => __("The shortcode key you provided is already used. Please try a new one.", "estatebud")]));
		} else {
			$data = $_POST;
			$data["shortcode_key"] = sanitize_title($data["shortcode_key"]);
			$data["shortcode_key"] = sanitize_title_with_dashes($data["shortcode_key"]);
			$data["shortcode_key"] = str_replace("-", "_", $data["shortcode_key"]);
			$data["create"] = time();
			$data["update"] = time();
			$values[$data["shortcode_key"]] = $data;
			update_option("estatebud_shortcode_settings", $values);
			echo json_encode(["status" => "success", "message" => __("Record Created.", "estatebud")]);
		}
	}
	wp_die();
}
add_action('wp_ajax_estatebud_save_shortcode', 'estatebud_save_shortcode');

/**
 * Ajax Method to delete the shortcode data
 *
 * @since 	 1.0.0
 */
function estatebud_delete_shortcode()
{
	if (current_user_can('manage_options') && !empty($_POST) && check_admin_referer('estatebud_delete_shortcode', 'shortcode_security')) {
		$values = get_option("estatebud_shortcode_settings", []);
		if (!is_array($values)) $values = [];
		$to_delete = sanitize_key($_POST["shortcode_key"]);
		$values = array_filter($values, function ($val, $key) use ($to_delete) {
			return $value["shortcode_key"] !== $to_delete && $key !== $to_delete;
		}, ARRAY_FILTER_USE_BOTH);
		update_option("estatebud_shortcode_settings", $values);
		echo json_encode(["status" => "success", "message" => __("Record Deleted.", "estatebud")]);
	}
	wp_die();
}
add_action('wp_ajax_estatebud_delete_shortcode', 'estatebud_delete_shortcode');

/**
 * Ajax Method to overwrite the shortcode data
 *
 * @since 	 1.0.0
 */
function estatebud_edit_shortcode()
{
	if (current_user_can('manage_options') && !empty($_POST) && check_admin_referer('estatebud_edit_shortcode', 'shortcode_security')) {
		$values = get_option("estatebud_shortcode_settings", []);
		if (!is_array($values)) $values = [];
		$data = $_POST;
		$data["shortcode_key"] = sanitize_title($data["shortcode_key"]);
		$data["shortcode_key"] = sanitize_title_with_dashes($data["shortcode_key"]);
		$data["shortcode_key"] = str_replace("-", "_", $data["shortcode_key"]);
		$data["update"] = time();
		unset($values[$data["old_key"]]);
		$values[$data["shortcode_key"]] = $data;
		update_option("estatebud_shortcode_settings", $values);
		echo json_encode(["status" => "success", "message" => __("Record Updated.", "estatebud")]);
	}
	wp_die();
}
add_action('wp_ajax_estatebud_edit_shortcode', 'estatebud_edit_shortcode');

/**
 * Ajax Method to fetch the shortcode data
 *
 * @since 	 1.0.0
 */
function estatebud_get_shortcode()
{
	if (current_user_can('manage_options') && !empty($_GET) && check_admin_referer('estatebud_get_shortcode', 'shortcode_security')) {
		$values = get_option("estatebud_shortcode_settings", []);
		if (!is_array($values) || !isset($values[$_GET["id"]])) return json_encode(["status" => "error", "message" => __("Record not found.", "estatebud")]);
		echo json_encode(["status" => "success", "data" => $values[$_GET["id"]]]);
	}
	wp_die();
}
add_action('wp_ajax_estatebud_get_shortcode', 'estatebud_get_shortcode');


/**
 * Ajax Method to save the search box widget data
 *
 * @since 	 1.0.0
 */
function estatebud_save_widget()
{
	if (current_user_can('manage_options') && !empty($_POST) && check_admin_referer('estatebud_save_widget', 'widget_security')) {
		$values = get_option("estatebud_widget_settings", []);
		if (!is_array($values)) $values = [];
		if (isset($values[$_POST["label"]])) {
			die(json_encode(["status" => "error", "message" => __("The widget key you provided is already used. Please try a new one.", "estatebud")]));
		} else {
			$data = $_POST;
			$data["widget_key"] = sanitize_title($data["label"]);
			$data["widget_key"] = sanitize_title_with_dashes($data["widget_key"]);
			$data["widget_key"] = str_replace("-", "_", $data["widget_key"]);
			$data["create"] = time();
			$data["update"] = time();
			$values[$data["widget_key"]] = $data;
			update_option("estatebud_widget_settings", $values);
			echo json_encode(["status" => "success", "message" => __("Record Created.", "estatebud")]);
		}
	}
	wp_die();
}
add_action('wp_ajax_estatebud_save_widget', 'estatebud_save_widget');

/**
 * Ajax Method to delete the search box widget data
 *
 * @since 	 1.0.0
 */
function estatebud_delete_widget()
{
	if (current_user_can('manage_options') && !empty($_POST) && check_admin_referer('estatebud_delete_widget', 'widget_security')) {
		$values = get_option("estatebud_widget_settings", []);
		if (!is_array($values)) $values = [];
		$to_delete = sanitize_key($_POST["widget_key"]);
		$values = array_filter($values, function ($val, $key) use ($to_delete) {
			return $value["label"] !== $to_delete && $key !== $to_delete;
		}, ARRAY_FILTER_USE_BOTH);
		update_option("estatebud_widget_settings", $values);
		echo json_encode(["status" => "success", "message" => __("Record Deleted.", "estatebud")]);
	}
	wp_die();
}
add_action('wp_ajax_estatebud_delete_widget', 'estatebud_delete_widget');

/**
 * Ajax Method to overwrite the search box widget data
 *
 * @since 	 1.0.0
 */
function estatebud_edit_widget()
{
	if (current_user_can('manage_options') && !empty($_POST) && check_admin_referer('estatebud_edit_widget', 'widget_security')) {
		$values = get_option("estatebud_widget_settings", []);
		if (!is_array($values)) $values = [];
		$data = $_POST;
		$data["widget_key"] = sanitize_title($data["label"]);
		$data["widget_key"] = sanitize_title_with_dashes($data["widget_key"]);
		$data["widget_key"] = str_replace("-", "_", $data["widget_key"]);
		$data["update"] = time();
		unset($values[$data["old_key"]]);
		$values[$data["widget_key"]] = $data;
		update_option("estatebud_widget_settings", $values);
		echo json_encode(["status" => "success", "message" => __("Record Updated.", "estatebud")]);
	}
	wp_die();
}
add_action('wp_ajax_estatebud_edit_widget', 'estatebud_edit_widget');

/**
 * Ajax Method to fetch the search box widget data
 *
 * @since 	 1.0.0
 */
function estatebud_get_widget()
{
	if (current_user_can('manage_options') && !empty($_GET) && check_admin_referer('estatebud_get_widget', 'widget_security')) {
		$values = get_option("estatebud_widget_settings", []);
		//var_dump($values);
		if (!is_array($values) || !isset($values[$_GET["id"]])) return json_encode(["status" => "error", "message" => __("Record not found.", "estatebud")]);
		echo json_encode(["status" => "success", "data" => $values[$_GET["id"]]]);
	}
	wp_die();
}
add_action('wp_ajax_estatebud_get_widget', 'estatebud_get_widget');

function estatebud_get_default_widget_template()
{
	echo file_get_contents(dirname(__FILE__) . "/includes/defaults/template-estatebud-searchbox.tpl");
	wp_die();
}
add_action('wp_ajax_estatebud_get_default_widget_template', 'estatebud_get_default_widget_template');

if(!function_exists("array_column"))
{
    function array_column($array,$column_name)
    {
        return array_map(function($element) use($column_name){return $element[$column_name];}, $array);
    }
}