<?php
class EstatebudAPI
{

    private $api_key  = null;
    private $version  = "v1";
    private $url      = "https://api.estatebud.com";
    private $locale   = null;

    public $is_valid  = false;
    public $auth_info = null;


    function __construct($api_key, $locale = "en_US")
    {
        $this->api_key = trim($api_key);
        if($api_key !== null){
            $this->is_valid = $this->check_authentication($api_key);
        }

        if($locale == 'el') {
            $locale = "el_GR";
        }

        $this->locale = str_replace("_","-",$locale);
    }

    /**
     * Checks whether the API key is valid/invalid
     *
     * @param string $api_key
     * @return bool Api key status
     */
    function check_authentication($api_key)
    {
        $result = $this->get_json("whois");
        if($result){
            if(isset($result->error)){
                return false;
            }else{
                $this->auth_info = $result;
                return true;
            }
        }else{
            echo __("Estatebud API Server: Communication Error", "estatebud");
            return false;
        }
    }

    /**
     * Endpoints
     */

    /**
     * Get Property Categories
     *
     * @return object property categories
     */

    public function get_property_categories(){
        return $this->get_json("getPropertyCategories", ["language" => $this->locale]);
    }

    /**
     * Get Property Types
     *
     * @return object property types
     */
    public function get_property_types(){
        return $this->get_json("getPropertyTypes", ["language" => $this->locale]);
    }

    /**
     * Get Property Conditions
     *
     * @return object property conditions
     */
    public function get_property_statuses(){
        return $this->get_json("getPropertyStatuses", ["language" => $this->locale]);
    }

    /**
     * Get Property Areas
     *
     * @param string $country
     * @return object property areas of country
     */
    public function get_property_areas($country = null){
        return $this->get_json("getAreas", ["country" => $country, "language" => $this->locale], true);
    }

    /**
     * Get Property Details availability + price within the date range
     *
     * @param int $id
     * @param date $date_from
     * @param date $date_to
     * @return object single property info
     */
    public function get_property($id, $date_from, $date_to){
        return $this->get_json("getProperty/$id", ["date_from" => $date_from, "date_to" => $date_to, "language" => $this->locale]);
    }

    /**
     * Get Custom Property Labels
     *
     * @return object property labels array
     */
    public function get_property_labels(){
        return $this->get_json("getPropertyLabels");
    }

    /**
     * Property Listing with filters
     *
     * @param int $property_id
     * @param int $area
     * @param string $category
     * @param string $type
     * @param string $label
     * @param int $city
     * @param date $date_from
     * @param date $date_to
     * @param bool $public
     * @param bool $featured
     * @param string $title_deed = yes, yes_share_of_land, final_approval, no
     * @param integer $bedrooms_min
     * @param integer $bedrooms_max
     * @param integer $bathrooms_min
     * @param integer $bathrooms_max
     * @param integer $covered_sqm_min
     * @param integer $covered_sqm_max
     * @param integer $price_min
     * @param integer $price_max
     * @param string $status = resale, key_ready, under_construction, off_plan
     * @param bool $sea_view
     * @param bool $sea_front
     * @param bool $swimming_pool
     * @param bool $walking_distance_to_beach
     * @param integer $rent_min
     * @param integer $rent_max
     * @param string $sort = availability, property_id (default), price, rent
     * @param string $sort_type = DESC, ASC
     * @param integer $offset
     * @param integer $limit
     * @param boolean $show_sold
     * @param boolean $show_rented
     * @return void
     */
    public function get_properties($property_id = null, $area = null, $category = null, $type = null, $label = null, $city = null, $date_from = null, $date_to = null, $public = null, $featured = null, $title_deed = null, $bedrooms_min = 0, $bedrooms_max = 999, $bathrooms_min = 0, $bathrooms_max = 999, $covered_m2_min = 0, $covered_m2_max = 999999, $covered_ft2_min = 0, $covered_ft2_max = 999999, $price_min = 0, $price_max = 999999999, $status = null , $sea_view = null, $sea_front = null, $swimming_pool = null, $walking_distance_to_beach = null, $rent_min = 0, $rent_max = 999999999, $sort = "property_id", $sort_type = "DESC",  $offset = 0, $limit = 12, $sold = true, $rented = true)
    {
        $ref = new ReflectionMethod("EstatebudAPI::get_properties");
        $parameters = $ref->getParameters();
        $query = [];
        foreach ($parameters as $parameter) {
            if ($parameter->name !== "offset" && $parameter->name !== "limit") {
                $value = ${$parameter->name};
                if ($value !== null && $value !== $parameter->getDefaultValue()) {
                    $query[$parameter->name] = ${$parameter->name};
                }
            }
        }
        $query["title_deed"] = !empty($query["title_deed"]);
        $query["limit"] = $offset . "," . $limit;
        $query["public"] = 1;
        $query["sort"] = $sort;
        $query["sort_type"] = $sort_type;
        $query["language"] = $this->locale;

        if(!$sold){
             $query["sold"] = "0";
        }else{
            unset($query["sold"]);
        }

        if(!$rented){
             $query["rented"] = "0";
        }else{
            unset($query["rented"]);
        }

        return $this->get_json("getProperties", $query);
    }

    /**
     * API Fetching method
     *
     * @param string    $endpoint    The API endpoint
     * @param array     $query       The query parameters
     * @return object                The query result
     */
    function get_json($endpoint, $query = [], $as_array = false)
    {
        $query = array_filter($query, function($d){
            $d = is_array($d) ? trim(implode(",", $d)) : $d;
            return is_string($d) ? strlen($d) : !empty($d);
        });
        $ch = curl_init();
        $url = "$this->url/$this->version/$endpoint" . (count($query) > 0 ? "?" . urldecode(http_build_query($query)) : "");
        // echo "<code style='white-space: pre-wrap; word-wrap: break-word;'>$url</code><br>";
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", "Authorization: Basic $this->api_key"));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $result = curl_exec($ch);
        curl_close($ch);
        if($result){
            // var_dump(json_decode($result, $as_array));
            return json_decode($result, $as_array);
        }else{
            return false;
        }
    }
}

?>