<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://estatebud.com
 * @since      1.0.0
 *
 * @package    Estatebud
 * @subpackage Estatebud/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Estatebud
 * @subpackage Estatebud/public
 * @author     Estatebud
 */
class Estatebud_Public
{
    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $id    The ID of this plugin.
     */
    private $id;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * The page type flag of details.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $is_details_page   Shows the page type is property details page.
     */
    private $is_details_page;

    /**
     * The page type flag of listings.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $is_listings_page   Shows the page type is property listings page.
     */
    private $is_listing_page;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $id         The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($id, $version)
    {
        $this->id = $id;
        $this->version = $version;
        $this->is_details_page = false;
        $this->is_listing_page = false;
        $this->register_shortcodes();
        $this->register_detail_shortcode();

        add_filter('document_title_parts', array($this, 'estatebud_filter_wp_title'));
        add_filter('template_redirect', array($this, 'estatebud_get_page_type'));
        add_filter('wp_head', array($this, 'add_estatebud_meta'));
        add_filter('the_title', array($this, 'estatebud_remove_title_from_properties'));

    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {
        wp_enqueue_style('font-awesome', '//maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css');
        wp_enqueue_style($this->id, plugin_dir_url(__FILE__) . 'css/estatebud-public.css', array(), $this->version, 'all');
        wp_enqueue_style($this->id . "-widget", plugin_dir_url(__FILE__) . 'css/estatebud-widget.css', array(), $this->version, 'all');
        wp_enqueue_style($this->id . "-icons", plugin_dir_url(__FILE__) . 'css/fonts/estatebud.css', array(), $this->version, 'all');
        wp_enqueue_style($this->id . "-theme-color", plugin_dir_url(__FILE__) . 'css/estatebud-dynamic.php', array(), null, 'all');
        wp_enqueue_style($this->id . "-checkbox", plugin_dir_url(__FILE__) . 'css/estatebud-checkbox.css', array(), $this->version, 'all');
        if ($this->is_listing_page) {
            wp_enqueue_style($this->id . "-select", plugin_dir_url(__FILE__) . 'css/select2.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-override", plugin_dir_url(__FILE__) . 'css/estatebud-select2.css', array(), $this->version, 'all');

        } else if ($this->is_details_page) {
            wp_enqueue_style($this->id . "-slider", plugin_dir_url(__FILE__) . 'css/slick.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-slider-override", plugin_dir_url(__FILE__) . 'css/estatebud-slick.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-share", plugin_dir_url(__FILE__) . 'css/jssocials.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-fancybox", plugin_dir_url(__FILE__) . 'css/jquery.fancybox.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-share-theme", plugin_dir_url(__FILE__) . 'css/jssocials-theme-minima.css', array(), $this->version, 'all');
        }
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {
        wp_enqueue_script($this->id, plugin_dir_url(__FILE__) . 'js/estatebud-public.js', array('jquery'), $this->version, false);
        if ($this->is_listing_page) {
            wp_enqueue_script($this->id . "-select", plugin_dir_url(__FILE__) . 'js/select2.full.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-deserialize", plugin_dir_url(__FILE__) . 'js/jquery.deserialize.js', array('jquery'), $this->version, false);
            wp_localize_script($this->id, 'estatebudL10n', array(
                'select2_empty' => __("Nothing Selected", "estatebud"),
                'select2_format' => __("Selected {count} of {total} items", "estatebud"),
                'show_less' => __("Show Less", "estatebud"),
                'show_more' => __("Show More", "estatebud")
            ));
        } else if ($this->is_details_page) {
            wp_enqueue_script($this->id . "-slider", plugin_dir_url(__FILE__) . 'js/slick.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-share", plugin_dir_url(__FILE__) . 'js/jssocials.min.js', array('jquery'), $this->version, false);

            if(get_option("estatebud_disable_fancybox", false) == false){
                if (!wp_script_is('jquery.fancybox.min.js')) {
                    wp_enqueue_script($this->id . "-fancybox", plugin_dir_url(__FILE__) . 'js/jquery.fancybox.min.js', array('jquery'), $this->version, false);
                }
            }
            if (trim(get_option("estatebud_gmaps_api_key", "")) !== "") {
                wp_enqueue_script($this->id . "-maps", "//maps.googleapis.com/maps/api/js?callback=estatebud_createMap&key=" . trim(get_option("estatebud_gmaps_api_key", "")), "", "", true);
            }
            wp_enqueue_script($this->id . "-recaptcha", "//www.google.com/recaptcha/api.js", "", "", true);
        }
        wp_localize_script($this->id, 'estatebud', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'is_details' => $this->is_details_page,
            'is_listing' => $this->is_listing_page
        ));
    }

    /**
     * If the page contains the detail shortcode, it rewrites the title with property number
     *
     * @param [type] $title_parts
     * @return void
     */
    public function estatebud_filter_wp_title($title_parts)
    {
        global $wp_query;
        if ($this->is_details_page) {
            $title_parts["title"] = sprintf(__("Property #%d", "estatebud"), isset($_GET["id"]) ? intval($_GET["id"]) : $wp_query->query["page"]);
            unset($title_parts["page"]);
            $wp_query->query["remove_title"] = true;
        } else if ($this->is_listing_page) {
            $wp_query->query["remove_title"] = true;
        }
        return $title_parts;
    }

    /**
     * Gets the page type (listing or details) prior to script/style enqueues.
     *
     * @return void
     */
    public function estatebud_get_page_type()
    {
        if (is_page()) {
            global $shortcode_tags, $wp_query;
            $estatebud_shortcodes = array_filter(array_keys($shortcode_tags), function ($shortcode) {
                return substr($shortcode, 0, 9) == "estatebud";
            });
            foreach ($estatebud_shortcodes as $shortcode) {
                if (has_shortcode($wp_query->post->post_content, $shortcode) !== false) {
                    if ($shortcode === "estatebud_detail") {
                        $this->is_details_page = true;
                        remove_action( 'wp_head', '_wp_render_title_tag', 1 );
                    } else {
                        $this->is_listing_page = true;
                    }
                    return;
                }
            }
        }
    }

    /**
     * Removes title from property pages
     */
    public function estatebud_remove_title_from_properties($title)
    {
        global $wp_query;
        if (isset($wp_query->query["remove_title"]) || $this->is_details_page) {
            if (!array_intersect(["widget", "wp_nav_menu"], array_column(debug_backtrace(), "function")))
                return "";
        }
        return $title;
    }

    /**
     * Registers the dynamic shortcodes
     *
     * @return void
     */
    public function register_shortcodes()
    {
        $shortcodes = get_option("estatebud_shortcode_settings", []);
        foreach ($shortcodes as $shortcode_key => $data) {
            add_shortcode("estatebud_" . $shortcode_key, function ($atts) use ($data) {
                $is_widget = isset($atts["is_widget"]);
                if ($is_widget) {
                    $property_limit = $atts["limit"];
                    $layout = $atts["layout"];
                }
                $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), get_locale());
                $title_deed = isset($data["title_deed"]) ? trim($data["title_deed"]) : "";
                if (!empty($title_deed)) {
                    $data["title_deed"] = 1;
                } else {
                    $data["title_deed"] = null;
                }
                $data["sold"] = get_option("estatebud_show_sold", true);
                $data["rented"] = get_option("estatebud_show_rented", true);

                // apply GET data from searchbox widget
                if(!$is_widget && isset($_GET["from"]) && $_GET["from"] == "searchbox"){
                    $request = $_GET;
				    if(!isset($request["category"])) $request["category"] = [];
				    if(!isset($request["status"])) $request["status"] = [];
				    if(!isset($request["area"])) $request["area"] = [];
				    if(!isset($request["type"])) $request["type"] = [];
					$data = array_merge($data, $request);
                }

                $estatebud_list = $this->call_user_func_assoc_array(array($estatebud_api, "get_properties"), $data);
                $request = $data;
                require dirname(__FILE__) . "/partials/estatebud-public-listing.php";
            });
        }
    }

    private function call_user_func_assoc_array($callback, $args){
        $ref = new ReflectionMethod(get_class($callback[0]) . "::" . $callback[1]);
        $parameters = $ref->getParameters();
        $params = [];
        foreach($parameters as $parameter){
            $param_name = $parameter->getName();
            $params[$param_name] = isset($args[$param_name]) ? $args[$param_name] : $parameter->getDefaultValue();
            if(is_array($params[$param_name])) $params[$param_name] = implode(",", $params[$param_name]);
        }
        return call_user_func_array($callback, $params);
    }

    /**
     * Registers the detail page shortcode
     *
     * @return void
     */
    public function register_detail_shortcode()
    {
        add_shortcode("estatebud_detail", function () {
            global $wp_query;
            if (isset($_GET["id"]) || isset($wp_query->query["page"])) {
                $property_id = isset($_GET["id"]) ? intval($_GET["id"]) : $wp_query->query["page"];
                if ($property_id) {
                    $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), isset($_GET["lang"]) ? $_GET["lang"] : get_locale());
                    if ($estatebud_api->is_valid) {
                        $details = $estatebud_api->get_property($property_id, null, null);
                        if (property_exists($details, "error")) {
                            echo __($details->error, $this->id);
                        } else {
                            $type = get_option("estatebud_details_type", "1");
                            $file = dirname(__FILE__) . "/partials/estatebud-public-details-$type.php";
                            if (file_exists($file)) {
                                require_once $file;
                            } else {
                                require_once dirname(__FILE__) . "/partials/estatebud-public-details-1.php";
                            }
                        }
                    }
                }
            }
        });
    }

    public function add_estatebud_meta()
    {
        global $wp_query;
        if ($this->is_details_page) {
            if (isset($_GET["id"]) || isset($wp_query->query["page"])) {
                $property_id = isset($_GET["id"]) ? intval($_GET["id"]) : $wp_query->query["page"];
                if ($property_id) {
                    $api_key = get_option("estatebud_api_key", null);
                    $estatebud_api = new EstatebudAPI($api_key, isset($_GET["lang"]) ? $_GET["lang"] : get_locale());
                    if ($estatebud_api->is_valid) {
                        $details = $estatebud_api->get_property($property_id, null, null);
                        if (property_exists($details, "error")) {
                            echo __($details->error, $this->id);
                        } else {
                            require_once dirname(__FILE__)."/partials/estatebud-public-details-functions.php";
                            /*echo "<pre>";
                            die(var_dump($details));*/
                            $sitename = get_bloginfo("name");
                            $property_id_string = sprintf(__("Property #%d", "estatebud"), $property_id);
                            $area2_name = $details->area2_name;
                            $area1_name = $details->area1_name;
                            $lang = get_locale();
                            $image = $details->images[0]->large;
                            $description = estatebud_get_translation_details($details->description);
                            $link = home_url(get_option("estatebud_property_page_id", null)) . "/" . intval($property_id, 10);
                            echo "<title>{$property_id_string}, {$area2_name}, {$area1_name} - {$sitename}</title>";
                            echo "<meta name='description' content='{$description}' />";
                            echo "<meta property=\"og:url\" content=\"{$link}\" />";
                            echo "<meta property=\"og:title\" content=\"Property #{$property_id}, {$area2_name}, {$area1_name}\" />";
                            echo "<meta property=\"og:description\" content=\"{$description}\" />";
                            echo "<meta property=\"og:image\" content=\"{$image}\" />";
                        }
                    }
                }
            }
        }
    }
}

function deregister_styles($wp_styles, $keytosearch)
{
    $deregister_styles = false;
    $keylist = [];
    foreach ($wp_styles->registered as $key => $registered) {
        if (strpos($registered->src, $keytosearch) > 0) {
            if (substr($key, 0, 9) != "estatebud") {
                $keylist[] = $key;
            } else {
                $deregister_styles = true;
            }
        }
    }
    if ($deregister_styles) {
        foreach ($keylist as $key) {
            wp_dequeue_style($key);
        }
    }
}

function deregister_scripts($wp_scripts, $keytosearch)
{
    $deregister_scripts = false;
    $keylist = [];
    foreach ($wp_scripts->registered as $key => $registered) {
        if (strpos($registered->src, $keytosearch) > 0) {
            if (substr($key, 0, 9) != "estatebud") {
                $keylist[] = $key;
            } else {
                $deregister_scripts = true;
            }
        }
    }
    if ($deregister_scripts) {
        foreach ($keylist as $key) {
            wp_dequeue_script($key);
        }
    }
}

function estatebud_inspect_scripts()
{
    global $wp_scripts;
    deregister_scripts($wp_scripts, "fancybox");
}
add_action('wp_print_scripts', 'estatebud_inspect_scripts', 100);

function estatebud_inspect_styles()
{
    global $wp_styles;
    deregister_styles($wp_styles, "fancybox");
}
add_action('wp_print_styles', 'estatebud_inspect_styles', 100);

function wpml_url_fix($languages)
{
    global $wp_query;
    if(isset($wp_query->query["page"])){
        $page = $wp_query->query["page"];
        foreach( $languages as $lang => $element ){
            $url = $languages[$lang]['url'];
            $paths = array_filter(explode("/", $url));
            if(end($paths) != $page){
                $languages[$lang]['url'] .= $page;
            }
        }
    }
    return $languages;
}
add_filter( 'icl_ls_languages', 'wpml_url_fix');